package com.yaohun.demonskills.skills;

import com.yaohun.demonskills.SkillsMain;
import com.yaohun.demonskills.component.DelayComponent;
import com.yaohun.demonskills.component.SoundComponent;
import com.yaohun.demonskills.component.damage.DamageComponent;
import com.yaohun.demonskills.component.dragoncore.ParticleSpawnCommponent;
import com.yaohun.demonskills.component.dragoncore.PlayerAnimationComponent;
import com.yaohun.demonskills.component.dragoncore.StandAnimationComponent;
import com.yaohun.demonskills.component.effects.StunPlayerComponent;
import com.yaohun.demonskills.component.particles.ParticlesComponent;
import com.yaohun.demonskills.core.Skill;
import com.yaohun.demonskills.data.PlayerData;
import com.yaohun.demonskills.manage.PlayerManager;
import com.yaohun.demonskills.target.ConeEnemiesSelector;
import com.yaohun.demonskills.target.SelfSelector;
import org.bukkit.Particle;
import org.bukkit.entity.Player;
import org.serverct.ersha.jd.AttributeAPI;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class Skill_崩地裂刺 {

    private static final String skillName = "崩地裂刺";
    private static final double cooldownTime = 35;
    private static final double cdReducePerLevel = 1.6;     // 每级减少1.2秒冷却
    private static final double minCooldown = 25.0;         // 最低冷却限制

    private static final Map<Integer, Integer> skillMagicMap = new HashMap<>();
    static {
        skillMagicMap.put(1, 1);
        skillMagicMap.put(2, 1);
        skillMagicMap.put(3, 2);
        skillMagicMap.put(4, 2);
        skillMagicMap.put(5, 3);
        skillMagicMap.put(6, 3);
        skillMagicMap.put(7, 4);
        skillMagicMap.put(8, 4);
    }

    public static Skill getSkillItemStack(int skillLevel){
        double realCooldown = Math.max(cooldownTime - cdReducePerLevel * (skillLevel - 1), minCooldown);
        int realNeedMagic = skillMagicMap.getOrDefault(skillLevel, 1);
        return new Skill(skillName, realNeedMagic, realCooldown, 2.2,new ArrayList<>());
    }

    public static Skill getSkillData(Player player) {
        String playerName = player.getName();
        PlayerManager playerManager = SkillsMain.getPlayerManager();
        PlayerData playerData = playerManager.getPlayerData(playerName);
        int skillLevel = playerData.getSkillLevel(skillName);
        double realCooldown = Math.max(cooldownTime - cdReducePerLevel * (skillLevel - 1), minCooldown);
        int realNeedMagic = skillMagicMap.getOrDefault(skillLevel, 1);
        double damageValue = AttributeAPI.getAttrData(player).getAttributeRandomValue("物理伤害");
        double damage = ((0.2 * skillLevel) * damageValue + 50) + damageValue;
        Skill skill = new Skill( // 技能 ID
                skillName,   // 技能名称
                realNeedMagic,      // 魔力需求
                realCooldown,      // 冷却时间
                2.2,
                Arrays.asList(
                        // 技能组件列表
                        new StunPlayerComponent(new SelfSelector(), 26L), // 定身2.0秒
                        new PlayerAnimationComponent("崩山击",34),
                        new StandAnimationComponent("战士技能", "崩山击", 1, 38),
                        new ParticleSpawnCommponent("战士暴雪/怒吼效果", 200, 2),
                        new SoundComponent("战士音效/怒吼音效.ogg", 0.5f, 1.8f),
                        new DelayComponent(15L,Arrays.asList(
                                new ParticlesComponent(Particle.LAVA,8,2,12),
                                new DamageComponent(damage,new ConeEnemiesSelector(4.5,180)),
                                new SoundComponent("战士音效/崩山击音效.ogg", 0.5f, 1.0f,10),
                                new DelayComponent(10L,Arrays.asList(
                                        new DamageComponent(damage,new ConeEnemiesSelector(6,180)),
                                        new SoundComponent("战士音效/崩山击音效.ogg", 0.5f, 1.0f,10)
                                )
                        ))
                )));
        playerData.setCooldownData(skillName, realCooldown);
        return skill;
    }

}
