package com.yaohun.demonskills.skills;

import com.yaohun.demonskills.SkillsMain;
import com.yaohun.demonskills.component.damage.DamageComponent;
import com.yaohun.demonskills.component.DelayComponent;
import com.yaohun.demonskills.component.PotionEffectComponent;
import com.yaohun.demonskills.component.SoundComponent;
import com.yaohun.demonskills.component.damage.DamageContinuedComponent;
import com.yaohun.demonskills.component.dragoncore.ParticleSpawnCommponent;
import com.yaohun.demonskills.component.dragoncore.ParticleSpawnContinuedComponent;
import com.yaohun.demonskills.component.dragoncore.PlayerAnimationComponent;
import com.yaohun.demonskills.component.dragoncore.StandAnimationComponent;
import com.yaohun.demonskills.component.effects.StunPlayerComponent;
import com.yaohun.demonskills.component.particles.ParticlesComponent;
import com.yaohun.demonskills.core.Skill;
import com.yaohun.demonskills.data.PlayerData;
import com.yaohun.demonskills.manage.PlayerManager;
import com.yaohun.demonskills.target.NearestEnemiesSelector;
import com.yaohun.demonskills.target.SelfSelector;
import org.bukkit.Particle;
import org.bukkit.entity.Player;
import org.bukkit.potion.PotionEffectType;
import org.serverct.ersha.jd.AttributeAPI;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class Skill_狱炎怒吼 {

    private static final String skillName = "狱炎怒吼";
    private static final double cooldownTime = 25;
    private static final double cdReducePerLevel = 1.2;     // 每级减少1.2秒冷却
    private static final double minCooldown = 18.0;         // 最低冷却限制

    private static final Map<Integer, Integer> skillMagicMap = new HashMap<>();
    static {
        skillMagicMap.put(2, 1);
        skillMagicMap.put(3, 1);
        skillMagicMap.put(4, 2);
        skillMagicMap.put(5, 2);
        skillMagicMap.put(6, 3);
        skillMagicMap.put(7, 3);
        skillMagicMap.put(8, 4);
    }

    public static Skill getSkillItemStack(int skillLevel){
        double realCooldown = Math.max(cooldownTime - cdReducePerLevel * (skillLevel - 1), minCooldown);
        int realNeedMagic = skillMagicMap.getOrDefault(skillLevel, 1);
        return new Skill(skillName, realNeedMagic, realCooldown, 1.5,new ArrayList<>());
    }

    public static Skill getSkillData(Player player) {
        String playerName = player.getName();
        PlayerManager playerManager = SkillsMain.getPlayerManager();
        PlayerData playerData = playerManager.getPlayerData(playerName);
        int skillLevel = playerData.getSkillLevel(skillName);
        double realCooldown = Math.max(cooldownTime - cdReducePerLevel * (skillLevel - 1), minCooldown);
        int realNeedMagic = skillMagicMap.getOrDefault(skillLevel, 1);
        double damageValue = AttributeAPI.getAttrData(player).getAttributeRandomValue("物理伤害");
        double damage = ((0.2 * skillLevel) * damageValue + 50) + damageValue;
        Skill skill = new Skill( // 技能 ID
                skillName,   // 技能名称
                realNeedMagic,      // 魔力需求
                realCooldown,      // 冷却时间
                2.0, // 核心技能冷却(冷却中无法施展其他技能 包括翻滚)
                Arrays.asList(
                        // 技能组件列表
                        new StunPlayerComponent(new SelfSelector(), 30L), // 定身1.5秒
                        new PlayerAnimationComponent("怒吼"),
                        new StandAnimationComponent("战士技能", "怒吼", 1, 39),
                        new SoundComponent("战士音效/怒吼音效.ogg", 1, 1),
                        new DelayComponent(5L, Arrays.asList(
                                new DamageComponent(damage,new NearestEnemiesSelector(3)),
                                new DamageContinuedComponent(damage*0.1,new NearestEnemiesSelector(4),5+skillLevel,15L,5),
                                new ParticleSpawnContinuedComponent("战士暴雪/怒吼效果",new NearestEnemiesSelector(4),5+skillLevel,15L,5),
                                new ParticleSpawnCommponent("战士暴雪/怒吼效果", 200, 1),
                                new ParticlesComponent(Particle.REDSTONE, 3, 1.5, 6 * 20),
                                new PotionEffectComponent(PotionEffectType.SPEED, 6 * 20, 1)
                        ))
                )
        );
        playerData.setCooldownData(skillName, realCooldown);
        return skill;
    }

}
